# -*- coding: utf-8 -*-
#
#  __init__.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-27.
#  Copyright 2012 Lars Yencken. All rights reserved.
#

"""
Supporting utilities for the anytop package.
"""

